@@ -2,10 +2,13 @@
 package Log::Log4perl;
 ##################################################
 
+END { local($?); Log::Log4perl::Logger::cleanup(); }
+
 use 5.006;
 use strict;
 use warnings;
 
+use Log::Log4perl::Util;
 use Log::Log4perl::Logger;
 use Log::Log4perl::Level;
 use Log::Log4perl::Config;
@@ -13,7 +16,7 @@ use Log::Log4perl::Appender;
 
 use constant _INTERNAL_DEBUG => 1;
 
-our $VERSION = '0.41';
+our $VERSION = '1.02';
 
    # set this to '1' if you're using a wrapper
    # around Log::Log4perl
@@ -56,6 +59,10 @@ our $JOIN_MSG_ARRAY_CHAR = '';
     #and XML Config unit tests
 our $DOM_VERSION_REQUIRED = '1.29'; 
 
+our $CHATTY_DESTROY_METHODS = 0;
+
+our $LOGDIE_MESSAGE_ON_STDERR = 1;
+our $LOGEXIT_CODE             = 1;
 
 ##################################################
 sub import {
@@ -75,6 +82,11 @@ sub import {
         $tags{'get_logger'} = 1;
     }
 
+    if(exists $tags{':no_extra_logdie_message'}) {
+        $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR = 0;
+        delete $tags{':no_extra_logdie_message'};
+    }
+
     if(exists $tags{get_logger}) {
         # Export get_logger into the calling module's 
 
@@ -105,42 +117,83 @@ sub import {
 
             # Define default logger object in caller's package
         my $logger = get_logger("$caller_pkg");
-        my $string = "\$${caller_pkg}::_default_logger = \$logger";
-        eval $string or die "$@";
-
+        ${$caller_pkg . '::_default_logger'} = $logger;
+        
             # Define DEBUG, INFO, etc. routines in caller's package
         for(qw(DEBUG INFO WARN ERROR FATAL)) {
             my $level   = $_;
             my $lclevel = lc($_);
             *{"$caller_pkg\::$_"} = sub { 
                 Log::Log4perl::Logger::init_warn() unless 
-                               $Log::Log4perl::Logger::INITIALIZED;
+                    $Log::Log4perl::Logger::INITIALIZED or
+                    $Log::Log4perl::Logger::NON_INIT_WARNED;
                 $logger->{$level}->($logger, @_, $level);
             };
         }
 
+            # Define LOGCROAK, LOGCLUCK, etc. routines in caller's package
+        for(qw(LOGCROAK LOGCLUCK LOGCARP LOGCONFESS)) {
+            my $method = "Log::Log4perl::Logger::" . lc($_);
+
+            *{"$caller_pkg\::$_"} = sub {
+                unshift @_, $logger;
+                goto &$method;
+            };
+        }
+
             # Define LOGDIE, LOGWARN
 
         *{"$caller_pkg\::LOGDIE"} = sub {
             Log::Log4perl::Logger::init_warn() unless 
-                           $Log::Log4perl::Logger::INITIALIZED;
+                    $Log::Log4perl::Logger::INITIALIZED or
+                    $Log::Log4perl::Logger::NON_INIT_WARNED;
             $logger->{FATAL}->($logger, @_, "FATAL");
-            CORE::die(Log::Log4perl::Logger::callerline(join '', @_));
+            $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ?
+                CORE::die(Log::Log4perl::Logger::callerline(join '', @_)) :
+                exit $Log::Log4perl::LOGEXIT_CODE;
+        };
+
+        *{"$caller_pkg\::LOGEXIT"} = sub {
+            Log::Log4perl::Logger::init_warn() unless 
+                    $Log::Log4perl::Logger::INITIALIZED or
+                    $Log::Log4perl::Logger::NON_INIT_WARNED;
+            $logger->{FATAL}->($logger, @_, "FATAL");
+            exit $Log::Log4perl::LOGEXIT_CODE;
         };
 
         *{"$caller_pkg\::LOGWARN"} = sub { 
             Log::Log4perl::Logger::init_warn() unless 
-                           $Log::Log4perl::Logger::INITIALIZED;
+                    $Log::Log4perl::Logger::INITIALIZED or
+                    $Log::Log4perl::Logger::NON_INIT_WARNED;
             $logger->{WARN}->($logger, @_, "WARN");
-            CORE::warn(Log::Log4perl::Logger::callerline(join '', @_));
+            $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ?
+            CORE::warn(Log::Log4perl::Logger::callerline(join '', @_)) :
+            exit $Log::Log4perl::LOGEXIT_CODE;
         };
     }
 
     if(exists $tags{':nowarn'}) {
-        $Log::Log4perl::Logger::INITIALIZED = 1;
+        $Log::Log4perl::Logger::NON_INIT_WARNED = 1;
         delete $tags{':nowarn'};
     }
 
+    if(exists $tags{':resurrect'}) {
+        my $FILTER_MODULE = "Filter::Util::Call";
+        if(! Log::Log4perl::Util::module_available($FILTER_MODULE)) {
+            die "$FILTER_MODULE required with :unhide" .
+                "(install from CPAN)";
+        }
+        eval "require $FILTER_MODULE" or die "Cannot pull in $FILTER_MODULE";
+        Filter::Util::Call::filter_add(
+            sub {
+                my($status);
+                s/^\s*###l4p// if
+                    ($status = Filter::Util::Call::filter_read()) > 0;
+                $status;
+                });
+        delete $tags{':resurrect'};
+    }
+
     if(keys %tags) {
         # We received an Option we couldn't understand.
         die "Unknown Option(s): @{[keys %tags]}";
@@ -167,6 +220,13 @@ sub reset { # Mainly for debugging/testing
     return Log::Log4perl::Logger->reset();
 }
 
+##################################################
+sub init_once { # Call init only if it hasn't been
+                # called yet.
+##################################################
+    init(@_) unless $Log::Log4perl::Logger::INITIALIZED;
+}
+
 ##################################################
 sub init { # Read the config file
 ##################################################
@@ -242,13 +302,20 @@ sub easy_init { # Initialize the root logger with a screen appender
             $app = Log::Log4perl::Appender->new(
                 "Log::Log4perl::Appender::Screen",
                 stderr => 0);
-        } elsif($logger->{file} =~ /^(>)?(>)?/) {
+        } else {
+            my $binmode;
+            if($logger->{file} =~ s/^(:.*?)>/>/) {
+                $binmode = $1;
+            }
+            $logger->{file} =~ /^(>)?(>)?/;
             my $mode = ($2 ? "append" : "write");
-            $logger->{file} =~ s/>+//g;
+            $logger->{file} =~ s/.*>+//g;
             $app = Log::Log4perl::Appender->new(
                 "Log::Log4perl::Appender::File",
                 filename => $logger->{file},
-                mode     => $mode);
+                mode     => $mode,
+                binmode  => $binmode,
+            );
         }
 
         my $layout = Log::Log4perl::Layout::PatternLayout->new(
@@ -259,6 +326,8 @@ sub easy_init { # Initialize the root logger with a screen appender
         $log->level($logger->{level});
         $log->add_appender($app);
     }
+
+    $Log::Log4perl::Logger::INITIALIZED = 1;
 }
 
 ##################################################
@@ -289,6 +358,90 @@ sub appenders {  # Get all defined appenders hashref
     return \%Log::Log4perl::Logger::APPENDER_BY_NAME;
 }
 
+##################################################
+sub appender_thresholds_adjust {  # Readjust appender thresholds
+##################################################
+        # If someone calls L4p-> and not L4p::
+    shift if $_[0] eq __PACKAGE__;
+    my($delta, $appenders) = @_;
+
+    if(defined $appenders) {
+            # Map names to objects
+        $appenders = [map { 
+                       die "Unkown appender: '$_'" unless exists
+                          $Log::Log4perl::Logger::APPENDER_BY_NAME{
+                            $_};
+                       $Log::Log4perl::Logger::APPENDER_BY_NAME{
+                         $_} 
+                      } @$appenders];
+    } else {
+            # Just hand over all known appenders
+        $appenders = [values %{Log::Log4perl::appenders()}] unless 
+            defined $appenders;
+    }
+
+        # Change all appender thresholds;
+    foreach my $app (@$appenders) {
+        my $old_thres = $app->threshold();
+        my $new_thres;
+        if($delta > 0) {
+            $new_thres = Log::Log4perl::Level::get_higher_level(
+                             $old_thres, $delta);
+        } else {
+            $new_thres = Log::Log4perl::Level::get_lower_level(
+                             $old_thres, -$delta);
+        }
+
+        $app->threshold($new_thres);
+    }
+}
+
+##################################################
+sub appender_by_name {  # Get an appender by name
+##################################################
+        # If someone calls L4p->appender_by_name and not L4p::appender_by_name
+    shift if $_[0] eq __PACKAGE__;
+
+    my($name) = @_;
+
+    if(exists $Log::Log4perl::Logger::APPENDER_BY_NAME{
+                $name}) {
+        return $Log::Log4perl::Logger::APPENDER_BY_NAME{
+                 $name}->{appender};
+    } else {
+        return undef;
+    }
+}
+
+##################################################
+sub eradicate_appender {  # Remove an appender from the system
+##################################################
+        # If someone calls L4p->... and not L4p::...
+    shift if $_[0] eq __PACKAGE__;
+    Log::Log4perl::Logger->eradicate_appender(@_);
+}
+
+##################################################
+sub infiltrate_lwp {  # 
+##################################################
+    no warnings qw(redefine);
+
+    my $l4p_wrapper = sub {
+        my($prio, @message) = @_;
+        $Log::Log4perl::caller_depth += 2;
+        get_logger(caller(1))->log($prio, @message);
+        $Log::Log4perl::caller_depth -= 2;
+    };
+
+    *LWP::Debug::trace = sub { 
+        $l4p_wrapper->($INFO, @_); 
+    };
+    *LWP::Debug::conns =
+    *LWP::Debug::debug = sub { 
+        $l4p_wrapper->($DEBUG, @_); 
+    };
+}
+
 1;
 
 __END__
@@ -299,6 +452,16 @@ Log::Log4perl - Log4j implementation for Perl
 
 =head1 SYNOPSIS
  
+        # Easy mode if you like it simple ...
+
+    use Log::Log4perl qw(:easy);
+    Log::Log4perl->easy_init($ERROR);
+
+    DEBUG "This doesn't go anywhere";
+    ERROR "This gets logged";
+
+        # ... or standard mode for more features:
+
     Log::Log4perl::init('/etc/log4perl.conf');
     
     --or--
@@ -328,7 +491,7 @@ Log::Log4perl - Log4j implementation for Perl
        
 =head1 ABSTRACT
 
-    Log::Log4perl provides a powerful logging API to your application
+    Log::Log4perl provides a powerful logging API for your application
 
 =head1 DESCRIPTION
 
@@ -462,28 +625,28 @@ we're using with a handle to the C<Logger>:
     }
 
 Instead, if a function/method wants a reference to the logger, it
-just calls the Logger's static C<get_logger()> method to obtain
+just calls the Logger's static C<get_logger($category)> method to obtain
 a reference to the I<one and only> possible logger object of
 a certain category.
 That's called a I<singleton> if you're a Gamma fan.
 
 How does the logger know
 which messages it is supposed to log and which ones to suppress?
-C<Log::Log4perl> works with inheritence: The config file above didn't 
+C<Log::Log4perl> works with inheritance: The config file above didn't 
 specify anything about C<My::MegaPackage>. 
 And yet, we've defined a logger of the category 
 C<My::MegaPackage>.
 In this case, C<Log::Log4perl> will walk up the class hierarchy
 (C<My> and then the we're at the root) to figure out if a log level is
 defined somewhere. In the case above, the log level at the root
-(root I<always> defines a log level, but not necessary an appender)
+(root I<always> defines a log level, but not necessarily an appender)
 defines that 
 the log level is supposed to be C<ERROR> -- meaning that I<debug>
 and I<info> messages are suppressed.
 
 =head2 Log Levels
 
-There's five predefined log levels: C<FATAL>, C<ERROR>, C<WARN>, C<INFO> 
+There are five predefined log levels: C<FATAL>, C<ERROR>, C<WARN>, C<INFO> 
 and C<DEBUG> (in descending priority). Your configured logging level
 has to at least match the priority of the logging message.
 
@@ -531,6 +694,11 @@ level, as derived from either the logger's category (or, in absence of
 that, one of the logger's parent's level setting) is 
 C<$WARN>, C<$ERROR> or C<$FATAL>.
 
+Also available are a series of more Java-esque functions which return
+the same values. These are of the format C<isI<Level>Enabled()>,
+so C<$logger-E<gt>isDebugEnabled()> is synonymous to 
+C<$logger-E<gt>is_debug()>.
+
 These level checking functions
 will come in handy later, when we want to block unnecessary
 expensive parameter construction in case the logging level is too
@@ -599,10 +767,12 @@ logger has one.
 C<Log::Log4perl> already comes with a standard set of appenders:
 
     Log::Log4perl::Appender::Screen
+    Log::Log4perl::Appender::ScreenColoredLevels
     Log::Log4perl::Appender::File
     Log::Log4perl::Appender::Socket
     Log::Log4perl::Appender::DBI
     Log::Log4perl::Appender::Synchronized
+    Log::Log4perl::Appender::RRDs
 
 to log to the screen, to files and to databases. 
 
@@ -785,7 +955,7 @@ debug level of the logger and others.
 
 There's currently two layouts defined in C<Log::Log4perl>: 
 C<Log::Log4perl::Layout::SimpleLayout> and
-C<Log::Log4perl::Layout::Patternlayout>:
+C<Log::Log4perl::Layout::PatternLayout>:
 
 =over 4 
 
@@ -795,7 +965,7 @@ formats a message in a simple
 way and just prepends it by the debug level and a hyphen:
 C<"$level - $message>, for example C<"FATAL - Can't open password file">.
 
-=item C<Log::Log4perl::PatternLayout> 
+=item C<Log::Log4perl::Layout::PatternLayout> 
 
 on the other hand is very powerful and 
 allows for a very flexible format in C<printf>-style. The format
@@ -1131,7 +1301,44 @@ real class name as an argument and all other methods can determine it
 via C<ref($self)>), so it shouldn't be a problem to get the right class
 every time.
 
-=head1 Custom Filters
+=head2 Initialize once and only once
+
+It's important to realize that Log::Log4perl gets initialized once and only
+once, typically at the start of a program or system. Calling C<init()>
+more than once will cause it to clobber the existing configuration and
+I<replace> it by the new one.
+
+If you're in a traditional CGI environment, where every request is
+handeled by a new process, calling C<init()> every time is fine. In
+persistent environments like C<mod_perl>, however, Log::Log4perl
+should be initialized either at system startup time (Apache offers
+startup handlers for that) or via
+
+        # Init or skip if already done
+    Log::Log4perl->init_once($conf_file);
+
+C<init_once()> is identical to C<init()>, just with the exception
+that it will leave a potentially existing configuration alone and 
+will only call C<init()> if Log::Log4perl hasn't been initialized yet.
+
+If you're just curious if Log::Log4perl has been initialized yet, the
+check
+
+    if(Log::Log4perl->initialized()) {
+        # Yes, Log::Log4perl has already been initialized
+    } else {
+        # No, not initialized yet ...
+    }
+
+can be used.
+
+If you're afraid that the components of your system are stepping on 
+each other's toes or if you are thinking that different components should
+initialize Log::Log4perl seperately, try to consolidate your system
+to use a centralized Log4perl configuration file and use 
+Log4perl's I<categories> to separate your components.
+
+=head2 Custom Filters
 
 Log4perl allows the use of customized filters in its appenders
 to control the output of messages. These filters might grep for
@@ -1146,7 +1353,7 @@ on how to use them.
 =head2 Performance
 
 The performance of Log::Log4perl calls obviously depends on a lot of things.
-but to give you a general idea, here's some rough numbers:
+But to give you a general idea, here's some rough numbers:
 
 On a Pentium 4 Linux box at 2.4 GHz, you'll get through
 
@@ -1173,6 +1380,10 @@ document in the Log::Log4perl distribution.
 
 =head1 Cool Tricks
 
+Here's a collection of useful tricks for the advanced C<Log::Log4perl> user.
+For more, check the the FAQ, either in the distribution 
+(L<Log::Log4perl::FAQ>) or on http://log4perl.sourceforge.net.
+
 =head2 Shortcuts
 
 When getting an instance of a logger, instead of saying
@@ -1186,16 +1397,24 @@ C<Log::Log4perl> into the current namespace:
     use Log::Log4perl qw(get_logger);
     my $logger = get_logger();
 
+Please note this difference: To obtain the root logger, please use
+C<get_logger("")>, call it without parameters (C<get_logger()>), you'll
+get the logger of a category named after the current package. 
+C<get_logger()> is equivalent to C<get_logger(__PACKAGE__)>.
+
 =head2 Alternative initialization
 
-Instead of having C<init()> read in a configuration file, you can 
+Instead of having C<init()> read in a configuration file by specifying
+a file name or passing it a reference to an open filehandle
+(C<Log::Log4perl-E<gt>init( \*FILE )>),
+you can 
 also pass in a reference to a string, containing the content of
 the file:
 
     Log::Log4perl->init( \$config_text );
 
 Also, if you've got the C<name=value> pairs of the configuration in
-a hash, you can just as well initialized C<Log::Log4perl> with
+a hash, you can just as well initialize C<Log::Log4perl> with
 a reference to it:
 
     my %key_value_pairs = (
@@ -1340,7 +1559,8 @@ username for a DBI appender:
 
 However, please note the difference between these code snippets and those
 used for user-defined conversion specifiers as discussed in
-L<Log::Log4perl::PatternLayout>: While the snippets above are run I<once>
+L<Log::Log4perl::Layout::PatternLayout>: 
+While the snippets above are run I<once>
 when C<Log::Log4perl::init()> is called, the conversion specifier
 snippets are executed I<each time> a message is rendered according to
 the PatternLayout.
@@ -1659,6 +1879,14 @@ the scenes. Also the keywords C<STDOUT> and C<STDERR> (no C<E<gt>> or
 C<E<gt>E<gt>>) are recognized, which will utilize and configure
 C<Log::Log4perl::Appender::Screen> appropriately.
 
+If a file appender receives Unicode strings, use
+
+    file => ":utf8> test.log"
+
+to establish a utf8 line discpline on the file, otherwise you'll get
+a 'wide character in print' warning message and probably not what
+you'd expect as output.
+
 The stealth loggers can be used in different packages, you just need to make
 sure you're calling the "use" function in every package you're using
 C<Log::Log4perl>'s easy services:
@@ -1690,8 +1918,9 @@ C<Log::Log4perl>'s easy services:
 As shown above, C<easy_init()> will take any number of different logger 
 definitions as hash references.
 
-Also, stealth loggers feature the functions C<LOGWARN()> and C<LOGDIE()>,
-combining a logging request with a subsequent Perl warn() or die()
+Also, stealth loggers feature the functions C<LOGWARN()>, C<LOGDIE()>,
+and C<LOGEXIT()>,
+combining a logging request with a subsequent Perl warn() or die() or exit()
 statement. So, for example
 
     if($all_is_lost) {
@@ -1704,6 +1933,10 @@ C<die()> (including the traditional output to STDERR) in any case afterwards.
 See L<"Log and die or warn"> for the similar C<logdie()> and C<logwarn()>
 functions of regular (i.e non-stealth) loggers.
 
+Similarily, C<LOGCARP()>, C<LOGCLUCK()>, C<LOGCROAK()>, and C<LOGCONFESS()>
+are provided in C<:easy> mode, facilitating the use of C<logcarp()>,
+C<logcluck()>, C<logcroak()>, and C<logconfess()> with stealth loggers.
+
 B<When using Log::Log4perl in easy mode, 
 please make sure you understand the implications of 
 L</"Pitfalls with Categories">>.
@@ -1811,6 +2044,85 @@ later on in the program.
 
 For details, please check L<Log::Log4perl::MDC>.
 
+=head2 Resurrecting hidden Log4perl Statements
+
+Sometimes scripts need to be deployed in environments without having
+Log::Log4perl installed yet. On the other hand, you dont't want to
+live without your Log4perl statements -- they're gonna come in
+handy later.
+
+So, just deploy your script with Log4perl statements commented out with the
+pattern C<###l4p>, like in
+
+    ###l4p DEBUG "It works!";
+    # ...
+    ###l4p INFO "Really!";
+
+If Log::Log4perl is available,
+use the C<:resurrect> tag to have Log4perl resurrect those burried 
+statements before the script starts running:
+
+    use Log::Log4perl qw(:resurrect :easy);
+
+    ###l4p Log::Log4perl->easy_init($DEBUG);
+    ###l4p DEBUG "It works!";
+    # ...
+    ###l4p INFO "Really!";
+
+This will have a source filter kick in and indeed print
+
+    2004/11/18 22:08:46 It works!
+    2004/11/18 22:08:46 Really!
+
+In environments lacking Log::Log4perl, just comment out the first line
+and the script will run nevertheless (but of course without logging):
+
+    # use Log::Log4perl qw(:resurrect :easy);
+
+    ###l4p Log::Log4perl->easy_init($DEBUG);
+    ###l4p DEBUG "It works!";
+    # ...
+    ###l4p INFO "Really!";
+
+because everything's a regular comment now. Alternatively, put the
+magic Log::Log4perl comment resurrection line into your shell's 
+PERL5OPT environment variable, e.g. for bash:
+
+    set PERL5OPT=-MLog::Log4perl=:resurrect,:easy
+    export PERL5OPT
+
+This will awaken the giant within an otherwise silent script like
+the following:
+
+    #!/usr/bin/perl
+
+    ###l4p Log::Log4perl->easy_init($DEBUG);
+    ###l4p DEBUG "It works!";
+
+=head2 Access defined appenders
+
+All appenders defined in the configuration file or via Perl code
+can be retrieved by the C<appender_by_name()> class method. This comes
+in handy if you want to manipulate or query appender properties after
+the Log4perl configuration has been loaded via C<init()>.
+
+=head2 Modify appender thresholds
+
+To conveniently adjust appender thresholds (e.g. because a script
+uses more_logging()), use
+
+       # decrease thresholds of all appenders
+    Log::Log4perl->appender_thresholds_adjust(-1);
+
+This will decrease the thresholds of all appenders in the system by
+one level, i.e. WARN becomes INFO, INFO becomes DEBUG, etc. To only modify 
+selected ones, use
+
+       # decrease thresholds of all appenders
+    Log::Log4perl->appender_thresholds_adjust(-1, ['AppName1', ...]);
+
+and pass the names of affected appenders in a ref to an array.
+
 =head1 Advanced configuration within Perl
 
 Initializing Log::Log4perl can certainly also be done from within Perl.
@@ -1876,7 +2188,7 @@ C<Log::Dispatch> objects to blindly log everything we send them
 want to call the shots and decide on when and what to log.
 
 The call to the appender's I<layout()> method specifies the format (as a
-previously created C<Log::Log4perl::PatternLayout> object) in which the
+previously created C<Log::Log4perl::Layout::PatternLayout> object) in which the
 message is being logged in the specified appender. 
 If you don't specify a layout, the logger will fall back to
 C<Log::Log4perl::SimpleLayout>, which logs the debug level, a hyphen (-)
@@ -1889,7 +2201,7 @@ Layouts are objects, here's how you create them:
 
         # create a flexible layout:
         # ("yyyy/MM/dd hh:mm:ss (file:lineno)> message\n")
-    my $pattern = Log::Log4perl::PatternLayout("%d (%F:%L)> %m%n");
+    my $pattern = Log::Log4perl::Layout::PatternLayout("%d (%F:%L)> %m%n");
 
 Every appender has exactly one layout assigned to it. You assign
 the layout to the appender using the appender's C<layout()> object:
@@ -1909,6 +2221,19 @@ the layout to the appender using the appender's C<layout()> object:
     $logger->debug("Blah");
         # => "2002/07/10 23:55:35 (test.pl:207)> Blah\n"
 
+It's also possible to remove appenders from a logger:
+
+    $logger->remove_appender($appender_name);
+
+will remove an appender, specified by name, from a given logger. 
+Please note that this does
+I<not> remove an appender from the system.
+
+To eradicate an appender from the system, 
+you need to call C<Log::Log4perl-E<gt>eradicate_appender($appender_name)>
+which will first remove the appender from every logger in the system
+and then will delete all references Log4perl holds to it.
+
 =head1 How about Log::Dispatch::Config?
 
 Tatsuhiko Miyagawa's C<Log::Dispatch::Config> is a very clever 
@@ -1978,6 +2303,49 @@ names to their Log::Log4perl::Appender object references.
 
 =back
 
+=head1 Dirty Tricks
+
+=over 4
+
+=item infiltrate_lwp()
+
+The famous LWP::UserAgent module isn't Log::Log4perl-enabled. Often, though,
+especially when tracing Web-related problems, it would be helpful to get
+some insight on what's happening inside LWP::UserAgent. Ideally, LWP::UserAgent
+would even play along in the Log::Log4perl framework.
+
+A call to C<Log::Log4perl-E<gt>infiltrate_lwp()> does exactly this. 
+In a very rude way, it pulls the rug from under LWP::UserAgent and transforms
+its C<debug/conn> messages into C<debug()> calls of loggers of the category
+C<"LWP::UserAgent">. Similarily, C<LWP::UserAgent>'s C<trace> messages 
+are turned into C<Log::Log4perl>'s C<info()> method calls.
+
+=item Suppressing 'duplicate' LOGDIE messages
+
+If a script with a simple Log4perl configuration uses logdie() to catch
+errors and stop processing, as in 
+
+    use Log::Log4perl qw(:easy) ;
+    Log::Log4perl->easy_init($DEBUG);
+    
+    shaky_function() or LOGDIE "It failed!";
+
+there's a cosmetic problem: The message gets printed twice:
+
+    2005/07/10 18:37:14 It failed!
+    It failed! at ./t line 12
+
+The obvious solution is to use LOGEXIT() instead of LOGDIE(), but there's
+also a special tag for Log4perl that suppresses the second message:
+
+    use Log::Log4perl qw(:no_extra_logdie_message);
+
+This causes logdie() and logcroak() to call exit() instead of die(). To
+modify the script exit code in these occasions, set the variable
+C<$Log::Log4perl::LOGEXIT_CODE> to the desired value, the default is 1.
+
+=back
+
 =head1 EXAMPLE
 
 A simple example to cut-and-paste and get started:
@@ -2015,7 +2383,7 @@ why we're now shipping Log::Log4perl with its own standard appenders
 and only if you wish to use additional ones, you'll have to go through
 the C<Log::Dispatch> installation process.
 
-Log::Log4perl needs C<Test::Simple>, C<Test::Harness> and C<File::Spec>, 
+Log::Log4perl needs C<Test::More>, C<Test::Harness> and C<File::Spec>, 
 but they already come with fairly recent versions of perl.
 If not, everything's automatically fetched from CPAN if you're using the CPAN 
 shell (CPAN.pm), because they're listed as dependencies.
@@ -2062,7 +2430,7 @@ http://www.perl.com/pub/a/2002/09/11/log4perl.html
 
 =item [2]
 
-Ceki Glc, "Short introduction to log4j",
+Ceki Gülcü, "Short introduction to log4j",
 http://jakarta.apache.org/log4j/docs/manual.html
 
 =item [3]
@@ -2099,13 +2467,19 @@ our
     Kevin Goess <cpan@goess.org>
 
     Contributors:
+    Hutton Davidson <Davidson.Hutton@ftid.com>
     Chris R. Donnelly <cdonnelly@digitalmotorworks.com>
     James FitzGibbon <james.fitzgibbon@target.com>
+    Dennis Gregorovic <dgregor@redhat.com>
     Paul Harrington <Paul-Harrington@deshaw.com>
     David Hull <hull@paracel.com>
+    Jeff Macdonald <jeff.macdonald@e-dialog.com>
+    Markus Peter <warp@spin.de>
     Brett Rann <brettrann@mail.com>
     Erik Selberg <erik@selberg.com>
     Aaron Straup Cope <asc@vineyard.net>
+    Lars Thegler <lars@thegler.dk>
+    David Viner <dviner@yahoo-inc.com>
     Mac Yang <mac@proofpoint.com>
 
 =head1 COPYRIGHT AND LICENSE
