@@ -6,22 +6,67 @@ use 5.006;
 use strict;
 use warnings;
 
+use Log::Log4perl;
 use Log::Log4perl::Level;
 use Log::Log4perl::Layout;
 use Log::Log4perl::Appender;
 use Log::Log4perl::Filter;
 use Carp;
 
+$Carp::Internal{"Log::Log4perl"}++;
+$Carp::Internal{"Log::Log4perl::Logger"}++;
+
 use constant _INTERNAL_DEBUG => 0;
 
     # Initialization
 our $ROOT_LOGGER;
 our $LOGGERS_BY_NAME = {};
 our %APPENDER_BY_NAME = ();
-our $INITIALIZED;
+our $INITIALIZED = 0;
+our $NON_INIT_WARNED;
 
 __PACKAGE__->reset();
 
+##################################################
+sub cleanup {
+##################################################
+    # warn "Logger cleanup";
+
+    # Delete all loggers
+    foreach my $loggername (keys %$LOGGERS_BY_NAME){
+        # warn "Logger delete: $loggername";
+        $LOGGERS_BY_NAME->{$loggername}->DESTROY();
+        delete $LOGGERS_BY_NAME->{$loggername};
+    }
+
+    # Delete the root logger
+    undef $ROOT_LOGGER;
+
+    # Delete all appenders
+    foreach my $appendername (keys %APPENDER_BY_NAME){
+        if (exists $APPENDER_BY_NAME{$appendername} &&
+            exists $APPENDER_BY_NAME{$appendername}->{appender}) {
+                # Destroy L4p::Appender
+            $APPENDER_BY_NAME{$appendername}->DESTROY();
+            delete $APPENDER_BY_NAME{$appendername}->{appender};
+        }
+        delete $APPENDER_BY_NAME{$appendername};
+    }
+    %APPENDER_BY_NAME   = ();
+
+    undef $INITIALIZED;
+}
+
+##################################################
+sub DESTROY {
+##################################################
+    warn "Destroying logger $_[0]" if $Log::Log4perl::CHATTY_DESTROY_METHODS;
+
+    for(keys %{$_[0]}) {
+        delete $_[0]->{$_};
+    }
+}
+
 ##################################################
 sub reset {
 ##################################################
@@ -39,6 +84,7 @@ sub reset {
     }
     %APPENDER_BY_NAME   = ();
     undef $INITIALIZED;
+    undef $NON_INIT_WARNED;
     Log::Log4perl::Appender::reset();
 
     #clear out all the existing appenders
@@ -195,7 +241,7 @@ sub generate_coderef {
     }
 
     my $code = <<EOL;
-    \$coderef = sub {
+    sub {
       my (\$logger)  = shift;
       my (\$level)   = pop;
       my \$message;
@@ -232,7 +278,8 @@ sub generate_coderef {
           \$appender->log(
               #these get passed through to Log::Dispatch
               { name    => \$appender_name,
-                level   => \$Log::Log4perl::Level::L4P_TO_LD{\$level},   
+                level   => \$Log::Log4perl::Level::L4P_TO_LD{
+                               \$level},   
                 message => \$message,
               },
               #these we need
@@ -250,7 +297,7 @@ sub generate_coderef {
 
 EOL
 
-    eval $code or die "$@";
+    $coderef = eval $code or die "$@";
 
     return $coderef;
 }
@@ -288,14 +335,13 @@ sub generate_is_xxx_coderef {
 ##################################################
     my($return_token) = @_;
 
-    my $coderef    = '';
-    my $watch_code = '';
+    my $coderef    = sub { $return_token };
 
     if (defined $Log::Log4perl::Config::WATCHER) {
 
         my $cond = generate_watch_conditional();
 
-        $watch_code = <<EOL;
+        my $watch_code = <<EOL;
         my(\$logger, \$subname) = \@_;
         if($cond) {
             Log::Log4perl->init_and_watch();
@@ -303,13 +349,13 @@ sub generate_is_xxx_coderef {
             return \$logger->\$subname();
         }
 EOL
-    }
 
-    my $code = <<EOL;
-    \$coderef = sub { $watch_code return $return_token; };
+        my $code = <<EOL;
+        \$coderef = sub { $watch_code return $return_token; };
 EOL
 
-    eval $code or die "$@";
+        eval $code or die "$@";
+    }
 
     return $coderef;
 }
@@ -334,6 +380,12 @@ sub generate_watch_code {
             Log::Log4perl->init_and_watch();
                        
             my \$methodname = lc(\$level);
+
+                # Bump up the caller level by two, since
+                # we've artifically introduced additional levels.
+            local(\$Log::Log4perl::caller_depth);
+            \$Log::Log4perl::caller_depth += 2;
+
             \$logger->\$methodname(\@_); # send the message
                                          # to the new configuration
             return;        #and return, we're done with this incarnation
@@ -514,6 +566,57 @@ sub add_appender {
     &reset_all_output_methods
                 unless $dont_reset_all;  # keep us from getting overworked
                                          # if it's  the config file calling us
+
+        # For chaining calls ...
+    return $appender;
+}
+
+##################################################
+sub remove_appender {
+##################################################
+    my($self, $appender_name, $dont_reset_all, $sloppy) = @_;
+
+    my %appender_names = map { $_ => 1 } @{$self->{appender_names}};
+    
+    if(!exists $appender_names{$appender_name}) {
+        die "No such appender: $appender_name" unless $sloppy;
+        return undef;
+    }
+
+    delete $appender_names{$appender_name};
+    $self->{num_appenders}--;
+    $self->{appender_names} = [sort keys %appender_names];
+
+    &reset_all_output_methods
+                unless $dont_reset_all; 
+}
+
+##################################################
+sub eradicate_appender {
+##################################################
+        # If someone calls Logger->... and not Logger::...
+    shift if $_[0] eq __PACKAGE__;
+
+    my($appender_name, $dont_reset_all) = @_;
+
+    return 0 unless exists 
+        $APPENDER_BY_NAME{$appender_name};
+
+        # Remove the given appender from all loggers
+        # and delete all references to it, causing
+        # its DESTROY method to be called.
+    foreach my $logger (values %$LOGGERS_BY_NAME){
+        $logger->remove_appender($appender_name, 0, 1);
+    }
+        # Also remove it from the root logger
+    $ROOT_LOGGER->remove_appender($appender_name, 0, 1);
+    
+    delete $APPENDER_BY_NAME{$appender_name};
+
+    &reset_all_output_methods
+                unless $dont_reset_all; 
+
+    return 1;
 }
 
 ##################################################
@@ -536,7 +639,7 @@ sub log {
     $_[0] = $LOGGERS_BY_NAME->{$_[0]->{category}} if 
         defined $Log::Log4perl::Config::WATCHER;
 
-    init_warn() unless $INITIALIZED;
+    init_warn() unless $INITIALIZED or $NON_INIT_WARNED;
 
     croak "priority $priority isn't numeric" if ($priority =~ /\D/);
 
@@ -629,6 +732,7 @@ sub create_log_level_methods {
                            "forgot to pass in a level string!");
   my $lclevel = lc($level);
   my $levelint = uc($level) . "_INT";
+  my $initial_cap = ucfirst($lclevel);
 
   no strict qw(refs);
 
@@ -638,17 +742,24 @@ sub create_log_level_methods {
   *{__PACKAGE__ . "::$lclevel"} = sub {
         print "$lclevel: ($_[0]->{category}/$_[0]->{level}) [@_]\n" 
             if _INTERNAL_DEBUG;
-        init_warn() unless $INITIALIZED;
-        $_[0]->{$level}->(@_, $level);
+        init_warn() unless $INITIALIZED or $NON_INIT_WARNED;
+        $_[0]->{$level}->(@_, $level) if defined $_[0]->{$level};
      };
 
   # Added these to have is_xxx functions as fast as xxx functions
   # -ms
 
   *{__PACKAGE__ . "::is_$lclevel"} = sub {
-      $_[0]->{"is_" . $level}->($_[0], "is_" . $lclevel);
+      $_[0]->{"is_" . $level}->($_[0], "is_" . $lclevel) if
+          defined $_[0]->{$level};
   };
   
+  # Add the isXxxEnabled() methods as identical to the is_xxx
+  # functions. - dviner
+  
+  *{__PACKAGE__ . "::is".$initial_cap."Enabled"} = 
+                           \&{__PACKAGE__ . "::is_$lclevel"};
+  
   use strict qw(refs);
 
   return 0;
@@ -665,7 +776,7 @@ sub init_warn {
     CORE::warn "Log4perl: Seems like no initialization happened. " .
                "Forgot to call init()?\n";
     # Only tell this once;
-    $INITIALIZED = 1;
+    $NON_INIT_WARNED = 1;
 }
 
 #######################################################
@@ -676,6 +787,7 @@ sub callerline {
   # my ($pack, $file, $line) = caller(2);
   # but if we every bury this further, it'll break. So we do this
   # little trick stolen and paraphrased from Carp/Heavy.pm
+  my($message) = @_;
 
   my $i = 0;
   my (undef, $localfile, undef) = caller($i++);
@@ -684,14 +796,18 @@ sub callerline {
     ($pack, $file, $line) = caller($i++);
   } while ($file && $file eq $localfile);
 
-  # now, create the return message
-  my $mess = " at $file line $line";
+  my $has_newline;
+
+  $has_newline++ if chomp $message;
+
+  $message .= " at $file line $line" if !$has_newline;
+
   # Someday, we'll use Threads. Really.
   if (defined &Thread::tid) {
     my $tid = Thread->self->tid;
-    $mess .= " thread $tid" if $tid;
+    $message .= " thread $tid" if $tid and !$has_newline;
   }
-  return (@_, $mess, "\n");
+  return ($message, "\n");
 }
 
 #######################################################
@@ -699,7 +815,6 @@ sub and_warn {
 #######################################################
   my $self = shift;
   my $msg = join("", @_[0 .. $#_]);
-  chomp $msg;
   CORE::warn(callerline($msg));
 }
 
@@ -708,7 +823,6 @@ sub and_die {
 #######################################################
   my $self = shift;
   my $msg = join("", @_[0 .. $#_]);
-  chomp $msg;
   die(callerline($msg));
 }
 
@@ -719,7 +833,9 @@ sub logwarn {
   if ($self->is_warn()) {
         # Since we're one caller level off now, compensate for that.
     $Log::Log4perl::caller_depth++;
-    $self->warn(@_);
+    my @chomped = @_;
+    chomp($chomped[-1]);
+    $self->warn(@chomped);
     $Log::Log4perl::caller_depth--;
     $self->and_warn(@_);
   }
@@ -732,11 +848,32 @@ sub logdie {
   if ($self->is_fatal()) {
         # Since we're one caller level off now, compensate for that.
     $Log::Log4perl::caller_depth++;
-    $self->fatal(@_);
+    my @chomped = @_;
+    chomp($chomped[-1]);
+    $self->fatal(@chomped);
     $Log::Log4perl::caller_depth--;
   }
-  # no matter what, we die... 'cuz logdie wants you to die.
-  $self->and_die(@_);
+
+  $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
+      $self->and_die(@_) : 
+        exit($Log::Log4perl::LOGEXIT_CODE);
+}
+
+##################################################
+sub logexit {
+##################################################
+  my $self = shift;
+
+  if ($self->is_fatal()) {
+        # Since we're one caller level off now, compensate for that.
+    $Log::Log4perl::caller_depth++;
+    my @chomped = @_;
+    chomp($chomped[-1]);
+    $self->fatal(@chomped);
+    $Log::Log4perl::caller_depth--;
+  }
+
+  exit $Log::Log4perl::LOGEXIT_CODE;
 }
 
 ##################################################
@@ -752,12 +889,15 @@ sub noop {
 sub logcluck {
 ##################################################
   my $self = shift;
+
+  local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+
   if ($self->is_warn()) {
     my $message = Carp::longmess(@_);
     foreach (split(/\n/, $message)) {
       $self->warn("$_\n");
     }
-    CORE::warn(noop($message));
+    Carp::cluck(noop($message));
   }
 }
 
@@ -765,14 +905,17 @@ sub logcluck {
 sub logcarp {
 ##################################################
   my $self = shift;
+
+  local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+
   if ($self->is_warn()) {
     my $message = Carp::shortmess(@_);
     foreach (split(/\n/, $message)) {
       $self->warn("$_\n");
     }
-    CORE::warn(noop($message));
+    Carp::carp(noop($message)) if $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR;
   }
-} 
+}
 
 ##################################################
 # croaks and confess are FATAL level
@@ -780,28 +923,38 @@ sub logcarp {
 sub logcroak {
 ##################################################
   my $self = shift;
+
+  local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+
   my $message = Carp::shortmess(@_);
   if ($self->is_fatal()) {
     foreach (split(/\n/, $message)) {
       $self->fatal("$_\n");
     }
   }
-  # again, we die no matter what
-  die(noop($message));
+
+  $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
+      Carp::croak(noop($message)) : 
+        exit($Log::Log4perl::LOGEXIT_CODE);
 }
 
 ##################################################
 sub logconfess {
 ##################################################
   my $self = shift;
+
+#local $Carp::CarpLevel = 2;
+
   my $message = Carp::longmess(@_);
   if ($self->is_fatal()) {
     foreach (split(/\n/, $message)) {
       $self->fatal("$_\n");
     }
   }
-  # again, we die no matter what
-  die(noop($message));
+
+  $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
+      die(noop($message)) :
+        exit($Log::Log4perl::LOGEXIT_CODE);
 }
 
 ##################################################
@@ -825,7 +978,10 @@ sub error_die {
     $self->error(@_);
     $Log::Log4perl::caller_depth--;
   }
-  $self->and_die(@_);
+
+  $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
+      $self->and_die(@_) :
+        exit($Log::Log4perl::LOGEXIT_CODE);
 }
 
 ##################################################
